#ifndef SOFTUSART_H
#define SOFTUSART_H
//------------------------------------------------------------------------------
#include "Serial_Pins.h"
#include "Serial_Prototypes.h"
//------------------------------------------------------------------------------
#define BAUDRATE    1200
#define uSec        1000000
#define bitTime     (uSec/BAUDRATE)
//------------------------------------------------------------------------------
void Serial_init()	
{
  PORTS_config();
  
  Tx          = 1;			   // Transmit pin is High in idle state
  Tx_dir      = 0;		       // Output
  Rx_dir      = 1;		       // Input
  INTCON      = 0x90;          // Enable Global + INT interrupt
  OPTION_REG  = 0xBF;          // Make INT as Negative edge triggered
}
//-----------------------------------------------------------------------------------------
char Serial_receive(void)
{
  Serial_rxStart();
  char data = Serial_read();
  Serial_rxStop(&data);

  return data;
}
//-----------------------------------------------------------------------------------------
void Serial_transmit(char character)
{
  Serial_txStart();
  Serial_write(character);
  Serial_txStop();	
}
//-----------------------------wait for start bit-------------------------------
void Serial_rxStart()
{
  while( Rx == 1 );

  __delay_us(bitTime);
  __delay_us(bitTime/2);        // Take sample value in the mid of bit duration
}
//------------------------------Check for stop bit------------------------------
void Serial_rxStop(char *error)
{
  if( Rx == 1 )                 // Stop bit should be high
  {
	__delay_us(bitTime/2); 
  }
  else                          // some error occurred !
	  {
	    __delay_us(bitTime/2);
        *error = 0;
	  }
}
//------------------------------Send Start Bit----------------------------------
void Serial_txStart()
{
  /*   
   Basic Logic
	   
   TX pin is usually high. A high to low bit is the starting bit and 
   a low to high bit is the ending bit. No parity bit. No flow control.
   Data is transmitted LSB first.*/

  Tx = 0;
  __delay_us(bitTime);
}
//---------------------------------Send Stop Bit--------------------------------
void Serial_txStop()
{
  Tx = 1;
  __delay_us(bitTime);
}
//-----------------------------------------------------------------------------------------
char Serial_read(void)
{
  char data = 0;

  for( char read = 0; read < 8; read++ )
  {
    data += (Rx << read);                    //LSB first
	__delay_us(bitTime);
  }

  return data;
}
//-----------------------------------------------------------------------------------------
void Serial_write(char data)
{
  for( char write = 0; write < 8; write++ )
  {
	Tx = ( (data>>write) & 0x01 ) ? 1:0;     //LSB first
	__delay_us(bitTime);
  }
}
//------------------------------------------------------------------------------
void Serial_string(char *string)
{
  while(*string)
  {
    Serial_transmit(*string++);
  }
}
//------------------------------------------------------------------------------
#endif